package fh.mobile.androidballmaze;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.UUID;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.content.Intent;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.PowerManager;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.bluetooth.BluetoothDevice;

public class MainActivity extends Activity implements SensorEventListener {

	
	private SensorManager mSensorManager;
	private Sensor mAccelerometer;

	
	// UUID 
    private static final UUID MY_UUID =
    	UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
	
	private byte xpos;
	private byte ypos;
	
	private float gravity[];
    
    private Button bt;
    
    private PowerManager.WakeLock wl;
    
    private boolean connected = false;
    
	// Local Bluetooth adapter
    private BluetoothAdapter mBluetoothAdapter = null;
    

    private ConnectThread connect;
    private ConnectedThread connectedThread;
    
    
    protected static final int SUCCESS_CONNECT = 0;
    protected static final int MESSAGE_WRITE = 1;
    
    Handler mHandler = new Handler(){
		
		@Override
		public void handleMessage(Message msg) {
			super.handleMessage(msg);
			
			switch(msg.what){
				case SUCCESS_CONNECT:

					Toast.makeText(getApplicationContext(), "CONNECTED", 0).show();
					connected = true;
					
					//Button Text ändern
					bt.setText("Disconnect");
					
					connectedThread = new ConnectedThread((BluetoothSocket)msg.obj);
					connectedThread.start();
					
					
					//auf gerade stellen
					byte[] command = new byte[2];
				      
				    command[0]=(byte) 127;
				    command[1]=(byte) 127;

				    connectedThread.write(command);

					break;

				case MESSAGE_WRITE:
                	bluetoothWrite();
					break;
			}
			
			
		};
		
	};
    

    // Intent request codes
    private static final int REQUEST_ENABLE_BT = 3;
    private static final int REQUEST_CONNECT_DEVICE = 1;
    
    ListView lv;
    
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		// Initializing the gravity vector to zero.
        gravity = new float[3];
        gravity[0] = 0;
        gravity[1] = 0;
        gravity[2] = 0;
		
		
		mSensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
		mAccelerometer = mSensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
		mSensorManager.registerListener(this, mAccelerometer, SensorManager.SENSOR_DELAY_GAME);
		

		// Get local Bluetooth adapter
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();


        // Initializing the "connect" button.
        // Register this an the OnClickListener.
        bt = (Button) findViewById(R.id.btn_connect);
        

    	bt.setText("Connect");
        bt.setOnClickListener(new OnClickListener() {
			
			public void onClick(View v) {
				// When not connected, start the DeviceListActivity that allows the user to select a device.
				// The activity will return a bluetoothdevice and on that return, the onActivityResult() (see below)
				// member function is called.
				

				if (!connected) {
					Intent serverIntent = new Intent(MainActivity.this, DeviceList.class);
					startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE);
				// When connected, close the bluetooth connection.
				} else {
					
					
					//auf gerade stellen
					byte[] command = new byte[2];
				      
				      //command[0]='c';
				      command[0]=(byte) 127;
				      command[1]=(byte) 127;

				      connectedThread.write(command);

					connect.cancel();

					bt.setText("Connect");
					connected = false;
				}
	
			}
		});
        	
    
        // WakeLock lässt Telefon aktiv bleiben
        PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
        wl = pm.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK, "My Tag");
        wl.acquire();
	
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
	
	
	
	@Override
    public void onStart() {
        super.onStart();
        
        if (mBluetoothAdapter == null) {
            // Device does not support Bluetooth
        }

        //bluetooth einschalten falls deaktiviert
        if (!mBluetoothAdapter.isEnabled()) {
            Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
        }
          
    }

    public void onReceive(Context context, Intent intent) {
	}

	protected void onResume() {
		super.onResume();
		mSensorManager.registerListener(this, mAccelerometer, SensorManager.SENSOR_DELAY_GAME);
	}
		
	protected void onPause() {
		super.onPause();
		
		mSensorManager.unregisterListener(this);

		if (connected){
			connect.cancel();
		}

	}
	

	@Override
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		// TODO Auto-generated method stub	
	}

	@Override
	public void onSensorChanged(SensorEvent event) {
		
		TextView txtX= (TextView)findViewById(R.id.txt_X);
		TextView txtY= (TextView)findViewById(R.id.txt_Y);

		float alpha = (float) 0.8;
		
		gravity[0] = alpha * gravity[0] + (1 - alpha) * event.values[0];
		gravity[1] = alpha * gravity[1] + (1 - alpha) * event.values[1];
		gravity[2] = alpha * gravity[2] + (1 - alpha) * event.values[2];
		
		// Normalize the gravity vector and rescale it so that every component fits one byte.
	    float size=(float) Math.sqrt(Math.pow(gravity[0], 2)+Math.pow(gravity[1], 2)+Math.pow(gravity[2], 2));
	    xpos = (byte) ((128*gravity[0]/size)+127);
	    ypos = (byte) ((128*gravity[1]/size)+127);

		txtX.setText(Integer.toString(xpos));
		txtY.setText(Integer.toString(ypos));

	}
	
	
	
	public void onActivityResult(int requestCode, int resultCode, Intent data) {

    	switch (requestCode) {
        case REQUEST_CONNECT_DEVICE:
        	if (resultCode == Activity.RESULT_OK) {
            	String address = data.getExtras().getString(DeviceList.EXTRA_DEVICE_ADDRESS);
            	System.out.println(address);
                BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);

                connect = new ConnectThread(device);
				connect.start();
 
            }
            break;

        }
        
    }
	
	
	private class ConnectThread extends Thread {
	    private final BluetoothSocket mmSocket;
	    //private final BluetoothDevice mmDevice;
	 
	    public ConnectThread(BluetoothDevice device) {
	        // Use a temporary object that is later assigned to mmSocket,
	        // because mmSocket is final
	        BluetoothSocket tmp = null;
	        //mmDevice = device;
	 
	        // Get a BluetoothSocket to connect with the given BluetoothDevice
	        try {
	            // MY_UUID is the app's UUID string, also used by the server code
	            tmp = device.createRfcommSocketToServiceRecord(MY_UUID);
	        } catch (IOException e) { }
	        mmSocket = tmp;
	    }
	 
	    public void run() {
	        // Cancel discovery because it will slow down the connection
	        //mBluetoothAdapter.cancelDiscovery();
	 
	        try {
	            // Connect the device through the socket. This will block
	            // until it succeeds or throws an exception
	            mmSocket.connect();
	        } catch (IOException connectException) {
	            // Unable to connect; close the socket and get out
	            try {
	                mmSocket.close();
	            } catch (IOException closeException) { }
	            return;
	        }
	 

	        mHandler.obtainMessage(SUCCESS_CONNECT, mmSocket).sendToTarget();
	          
	    }
	 
	    /** Will cancel an in-progress connection, and close the socket */
	    public void cancel() {
	        try {
	            mmSocket.close();
	        } catch (IOException e) { }
	    }
	    
 
	}
	

	public class ConnectedThread extends Thread {
	    private final BluetoothSocket mmSocket;
	    private final InputStream mmInStream;
	    private final OutputStream mmOutStream;
	 

	    public ConnectedThread(BluetoothSocket socket) {

	    	mmSocket = socket;
	        InputStream tmpIn = null;
	        OutputStream tmpOut = null;
	 
	        // Get the input and output streams, using temp objects because
	        // member streams are final
	        try {
	            tmpIn = socket.getInputStream();
	            tmpOut = socket.getOutputStream();
	        } catch (IOException e) { }
	 
	        mmInStream = tmpIn;
	        mmOutStream = tmpOut;
	    }
	 
	    public void run() {
	        byte[] buffer = new byte[1024];  // buffer store for the stream
	        int bytes; // bytes returned from read()

	        // Keep listening to the InputStream until an exception occurs
	        while (true) {
	            try {

	            	// Read from the InputStream
	                bytes = mmInStream.read(buffer);
	                
	                mHandler.obtainMessage(MESSAGE_WRITE, bytes, -1, buffer).sendToTarget();
	                //mHandler.obtainMessage(MESSAGE_WRITE).sendToTarget();

	                // Send the obtained bytes to the UI activity
	                //mHandler.obtainMessage(MESSAGE_READ, bytes, -1, buffer)
	                        //.sendToTarget();
	            } catch (IOException e) {
	                break;
	            }
	        }
	    }
	 
	    /* Call this from the main activity to send data to the remote device */
	    public void write(byte[] bytes) {
	        try {
	            mmOutStream.write(bytes);
	        } catch (IOException e) { }
	    }
	 
	    /* Call this from the main activity to shutdown the connection */
	    public void cancel() {
	        try {
	            mmSocket.close();
	        } catch (IOException e) { }
	    }
	}

	//public void bluetoothWrite(int bytes, byte[] buffer) {
	public void bluetoothWrite() {
		
		
		//auf gerade stellen
		byte[] command = new byte[2];
	      

	      command[0]=(byte) ypos;
	      command[1]=(byte) xpos;
	      
	      	// Create temporary object
	        ConnectedThread r;
	      
	      synchronized (this) {
	            //if (mState != STATE_CONNECTED) return;
	            r = connectedThread;
	        }
	        // Perform the write unsynchronized
	        r.write(command);


	}
	

}